from os import getcwd
from pathlib import Path
from typing import Literal, Union

import aiofiles
import jinja2

from .browser import get_new_page

TEMPLATES_PATH = str(Path(__file__).parent / 'templates')

env = jinja2.Environment(
    extensions=['jinja2.ext.loopcontrols'],
    loader=jinja2.FileSystemLoader(TEMPLATES_PATH),
    enable_async=True,
)


async def text_to_pic(
    text: str,
    css_path: str = '',
    width: int = 500,
    type: Literal['jpeg', 'png'] = 'png',
    quality: Union[int, None] = None,
) -> bytes:
    """多行文本转图片

    Args:
        text (str): 纯文本, 可多行
        css_path (str, optional): css文件
        width (int, optional): 图片宽度，默认为 500
        type (Literal["jpeg", "png"]): 图片类型, 默认 png
        quality (int, optional): 图片质量 0-100 当为`png`时无效

    Returns:
        bytes: 图片, 可直接发送
    """
    template = env.get_template('text.html')

    return await html_to_pic(
        template_path=f'file://{css_path if css_path else TEMPLATES_PATH}',
        html=await template.render_async(
            text=text,
            css=await read_file(css_path) if css_path else await read_tpl('text.css'),
        ),
        viewport={'width': width, 'height': 10},
        type=type,
        quality=quality,
    )


# async def read_md(md_path: str) -> str:
#     async with aiofiles.open(str(Path(md_path).resolve()), mode="r") as f:
#         md = await f.read()
#     return markdown.markdown(md)


async def read_file(path: str) -> str:
    async with aiofiles.open(path, mode='r') as f:
        return await f.read()


async def read_tpl(path: str) -> str:
    return await read_file(f'{TEMPLATES_PATH}/{path}')


async def template_to_html(
    template_path: str,
    template_name: str,
    **kwargs,
) -> str:
    """使用jinja2模板引擎通过html生成图片

    Args:
        template_path (str): 模板路径
        template_name (str): 模板名
        **kwargs: 模板内容

    Returns:
        str: html
    """

    template_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(template_path),
        enable_async=True,
    )
    template = template_env.get_template(template_name)

    return await template.render_async(**kwargs)


async def html_to_pic(  # noqa: PLR0913
    html: str,
    wait: int = 0,
    template_path: str = f'file://{getcwd()}',  # noqa: B008
    type: Literal['jpeg', 'png'] = 'png',
    quality: Union[int, None] = None,
    use_browser: str = 'chromium',
    **kwargs,
) -> bytes:
    """html转图片

    Args:
        html (str): html文本
        wait (int, optional): 等待时间. Defaults to 0.
        template_path (str, optional): 模板路径 如 "file:///path/to/template/"
        type (Literal["jpeg", "png"]): 图片类型, 默认 png
        quality (int, optional): 图片质量 0-100 当为`png`时无效
        **kwargs: 传入 page 的参数

    Returns:
        bytes: 图片, 可直接发送
    """
    # logger.debug(f"html:\n{html}")
    if 'file:' not in template_path:
        raise Exception('template_path 应该为 file:///path/to/template')
    async with get_new_page(use_browser, **kwargs) as page:
        await page.goto(template_path)
        await page.set_content(html, wait_until='networkidle')
        await page.wait_for_timeout(wait)
        img_raw = await page.screenshot(
            full_page=True,
            type=type,
            quality=quality,
        )
    return img_raw


async def template_to_pic(  # noqa: PLR0913
    template_path: str,
    template_name: str,
    templates: dict,
    pages: dict = {  # noqa: B006
        'viewport': {'width': 500, 'height': 10},
        'base_url': f'file://{getcwd()}',  # noqa: B008
    },
    wait: int = 0,
    type: Literal['jpeg', 'png'] = 'png',
    quality: Union[int, None] = None,
) -> bytes:
    """使用jinja2模板引擎通过html生成图片

    Args:
        template_path (str): 模板路径
        template_name (str): 模板名
        templates (dict): 模板内参数 如: {"name": "abc"}
        pages (dict): 网页参数 Defaults to
            {"base_url": f"file://{getcwd()}", "viewport": {"width": 500, "height": 10}}
        wait (int, optional): 网页载入等待时间. Defaults to 0.
        type (Literal["jpeg", "png"]): 图片类型, 默认 png
        quality (int, optional): 图片质量 0-100 当为`png`时无效

    Returns:
        bytes: 图片 可直接发送
    """

    template_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(template_path),
        enable_async=True,
    )
    template = template_env.get_template(template_name)

    return await html_to_pic(
        template_path=f'file://{template_path}',
        html=await template.render_async(**templates),
        wait=wait,
        type=type,
        quality=quality,
        **pages,
    )


async def capture_element(
    url: str,
    element: str,
    timeout: float = 0,
    type: Literal['jpeg', 'png'] = 'png',
    quality: Union[int, None] = None,
    **kwargs,
) -> bytes:
    async with get_new_page(**kwargs) as page:
        await page.goto(url, timeout=timeout)
        img_raw = await page.locator(element).screenshot(
            type=type,
            quality=quality,
        )
    return img_raw



import os
import re
import shutil
import traceback
from asyncio.exceptions import TimeoutError

import json_tools
import orjson
from httpx import ConnectTimeout, PoolTimeout, TimeoutException
from loguru import logger

from ..config import hikari_config
from ..HttpClient_Pool import (
    get_client_default,
    get_client_wg,
    get_client_yuyuko,
    recreate_client_default,
    recreate_client_wg,
    recreate_client_yuyuko,
)
from ..model import Hikari_Model
from .publicAPI import get_AccountIdByName

api_list = {
    'asia': 'https://api.worldofwarships.asia/wows/ships/stats/',
    'eu': 'https://api.worldofwarships.eu/wows/ships/stats/',
    'na': 'https://api.worldofwarships.com/wows/ships/stats/',
    'cn': 'https://vortex.wowsgame.cn/api/accounts/',
    'ru': 'https://vortex.korabli.su/api/accounts/',
}
seach_ship_url = 'https://api.wows.shinoaki.com/public/wows/encyclopedia/ship/info'


async def get_latest_info(server, account_id):
    try:
        if server not in ['cn', 'ru']:
            client_wg = await get_client_wg()
            params = {
                'application_id': '907d9c6bfc0d896a2c156e57194a97cf',
                'account_id': account_id,
                'extra': 'rank_solo',
            }
            resp = await client_wg.get(api_list[server], params=params)
            result = orjson.loads(resp.content)
        else:
            client_default = await get_client_default()
            url = f'{api_list[server]}{account_id}/ships/pvp/'
            resp = await client_default.get(url)
            result = orjson.loads(resp.content)
        if not result:
            raise Exception
        return result
    except PoolTimeout:
        logger.error('连接池错误，尝试重新创建')
        await recreate_client_default()
        await recreate_client_wg()
        return None
    except TimeoutException:
        logger.error(f'获取{account_id}实时战绩超时，可能是网络原因')
        return None
    except Exception:
        logger.error(traceback.format_exc())
        return None


def get_last_info(account_id):
    try:
        listen_data_path = f'{hikari_config.game_path}/account_data'
        if not os.path.exists(listen_data_path):
            os.mkdir(listen_data_path)
        with open(f'{listen_data_path}/{account_id}.json', 'rb') as f:
            last_info = orjson.loads(f.read())
        return last_info
    except Exception:
        print(traceback.format_exc())


def write_latest_info(account_id, data):
    try:
        listen_data_path = f'{hikari_config.game_path}/account_data'
        if not os.path.exists(listen_data_path):
            os.mkdir(listen_data_path)
        with open(f'{listen_data_path}/{account_id}.json', 'w', encoding='utf-8') as f:
            f.write(orjson.dumps(data).decode())
    except Exception:
        print(traceback.format_exc())


def jsonDiff(a, b):
    result = json_tools.diff(a, b)
    return result


def get_config():
    listen_data_path = f'{hikari_config.game_path}/account_data'
    if not os.path.exists(listen_data_path):
        os.mkdir(listen_data_path)
    listen_config_path = f'{hikari_config.game_path}/listen_config.json'
    if not os.path.exists(listen_config_path):
        with open(listen_config_path, 'w') as f:
            f.write(orjson.dumps({}).decode())
    with open(listen_config_path, 'rb') as f:
        config = orjson.loads(f.read())
    return config


def write_config(config):
    listen_data_path = f'{hikari_config.game_path}/account_data'
    if not os.path.exists(listen_data_path):
        os.mkdir(listen_data_path)
    listen_config_path = f'{hikari_config.game_path}/listen_config.json'
    if not os.path.exists(listen_config_path):
        with open(listen_config_path, 'w') as f:
            f.write(orjson.dumps({}).decode())
    with open(listen_config_path, 'w', encoding='utf-8') as f:
        f.write(orjson.dumps(config).decode())
    return config


async def get_diff_ship(hikari: Hikari_Model):  # noqa: PLR0915
    """查询最新一轮监控信息"""
    try:
        listen_data_path = f'{hikari_config.game_path}/account_data'
        config = get_config()
        account_id_list, account_list, send_list = [], [], []
        # 获取监控列表，去重
        for _group_id, group_config in config.items():
            for each in group_config:
                if each['account_id'] not in account_id_list:
                    account_id_list.append(each['account_id'])
                    account_list.append(each.copy())
        logger.info('开始获取监控信息')
        for account in account_list:
            # 不存在记录本轮先创建
            if not os.path.exists(f"{listen_data_path}/{account['account_id']}.json"):
                latest_data = await get_latest_info(account['server'], account['account_id'])
                if not latest_data or not latest_data['status'] == 'ok':
                    continue
                write_latest_info(account['account_id'], latest_data)
                continue
            # 存在历史记录，拉取最新战绩判断是否有差异
            latest_data = await get_latest_info(account['server'], account['account_id'])
            if not latest_data or not latest_data['status'] == 'ok':
                continue
            last_data = get_last_info(account['account_id'])
            write_latest_info(account['account_id'], latest_data)
            diff = jsonDiff(last_data, latest_data)
            if diff:
                compare_list = []
                for each in diff:
                    if 'replace' in each:
                        value = each['value'] - each['prev']
                        info = {'path': each['replace'], 'value': value}
                        compare_list.append(info.copy())
                    if 'add' in each:
                        value = each['value']
                        info = {'path': each['add'], 'value': value}
                        compare_list.append(info.copy())
                logger.info(orjson.dumps(compare_list))

                battles, win, loss, damage, shipId, match_count = 0, 0, 0, 0, 0, 0
                ship_name_cn = ''
                for each in compare_list:
                    if '/pvp/wins' in each['path']:
                        win += each['value']
                    if '/pvp/losses' in each['path']:
                        loss += each['value']
                    if 'pvp/damage_dealt' in each['path']:
                        damage += each['value']
                    if account['server'] not in ['cn', 'ru']:
                        match = re.search(f"^/data/{account['account_id']}/(.*?)/pvp/battles$", each['path'])
                    else:
                        match = re.search(f"^/data/{account['account_id']}/statistics/(.*?)/pvp/battles_count$", each['path'])
                    if match:
                        match_count += 1
                        if match_count > 5:
                            break
                        battles += each['value']
                        if account['server'] not in ['cn', 'ru']:
                            index = int(match.group(1))
                            shipId = int(latest_data['data'][str(account['account_id'])][index]['ship_id'])
                        else:
                            shipId = int(match.group(1))
                        params = {'shipId': shipId}
                        client_yuyuko = await get_client_yuyuko()
                        resp = await client_yuyuko.get(seach_ship_url, params=params)
                        result = orjson.loads(resp.content)
                        if result['code'] == 200:
                            ship_name_cn = ship_name_cn + result['data']['nameCn'] + ','
                if match_count > 5:
                    break
                # 构建消息
                for _group_id, group_config in config.items():
                    for each in group_config:
                        nick_name = each['nick_name']
                        if account['account_id'] == each['account_id']:
                            if battles == 1:
                                msg = f'喜报：\n{nick_name}刚刚输了一场对局\n使用{ship_name_cn}伤害{damage}'
                                if win:
                                    msg = f'悲报：\n{nick_name}刚刚赢了一场对局\n使用{ship_name_cn}伤害{damage}'
                            elif battles >= 2:
                                msg = f'{nick_name}刚刚完成了{battles}场对局\n胜{win}负{loss}平{battles-win-loss}\n使用{ship_name_cn}\n总伤害{damage}'
                            else:
                                continue
                            send_param = {'group_id': _group_id, 'msg': msg, 'type': 'text'}
                            send_list.append(send_param.copy())
        if not send_list:
            return hikari.failed('没有新的监控战绩')
        return hikari.success(send_list)
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return hikari.error('请求超时了，请过会儿再尝试哦~')
    except PoolTimeout:
        await recreate_client_yuyuko()
        return hikari.error('连接池异常，请尝试重新查询~')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')


async def get_listen_list(hikari: Hikari_Model):
    """获取本群监控列表"""
    try:
        config = get_config()
        group_id = hikari.UserInfo.GroupId
        msg = '本群监控列表\n'
        flag = 1
        if str(group_id) in config:
            for each in config[str(group_id)]:
                msg += f"{flag}：账号:{each['account_id']},备注:{each['nick_name']}\n"
                flag += 1
        else:
            msg = '当前群监控列表为空，请联系机器人搭建者添加监控'
        return hikari.success(msg)
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')


async def add_listen_list(hikari: Hikari_Model):
    try:
        if hikari.Status == 'init':
            if hikari.Input.Search_Type == 3:
                hikari.Input.AccountId = await get_AccountIdByName(hikari.Input.Server, hikari.Input.AccountName)
                if not isinstance(hikari.Input.AccountId, int):
                    return hikari.error(f'{hikari.Input.AccountId}')
        else:
            return hikari.error('当前请求状态错误')
        add_param = {'server': hikari.Input.Server, 'account_id': hikari.Input.AccountId, 'nick_name': hikari.Input.Command_List[1]}
        config = get_config()
        group_id = hikari.UserInfo.GroupId
        if not group_id:
            return hikari.error('不支持私聊')
        if group_id in config:
            group_list = config[group_id]
        else:
            group_list = []
        group_list.append(add_param)
        config[group_id] = group_list
        write_config(config)
        return hikari.success('添加成功')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')


async def delete_listen_list(hikari: Hikari_Model):
    try:
        config = get_config()
        group_id = hikari.UserInfo.GroupId
        if not group_id:
            return hikari.error('不支持私聊')
        if group_id in config:
            group_list: list = config[group_id]
        else:
            return hikari.error('当前群没有监控列表')
        if hikari.Input.Select_Index > len(group_list):
            return hikari.error('请确认序号是否小于监控列表')

        group_list.pop(hikari.Input.Select_Index - 1)
        config[group_id] = group_list
        if len(config[group_id]) == 0:
            config.pop(group_id)
        write_config(config)
        return hikari.success('删除成功')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')


async def reset_config(hikari: Hikari_Model):
    try:
        listen_data_path = f'{hikari_config.game_path}/account_data'
        shutil.rmtree(listen_data_path, ignore_errors=True)
        os.mkdir(listen_data_path)
        listen_config_path = f'{hikari_config.game_path}/listen_config.json'
        os.remove(listen_config_path)
        with open(listen_config_path, 'w') as f:
            f.write(orjson.dumps({}).decode())
        return hikari.success('重置监控配置成功')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.success('重置监控配置失败')



import asyncio
import gzip
import traceback
from asyncio.exceptions import TimeoutError
from base64 import b64encode
from typing import List

import orjson
from bs4 import BeautifulSoup
from httpx import ConnectTimeout, PoolTimeout
from loguru import logger

from ..config import hikari_config
from ..data_source import number_url_homes
from ..HttpClient_Pool import (
    get_client_default,
    get_client_wg,
    get_client_yuyuko,
    recreate_client_default,
    recreate_client_wg,
    recreate_client_yuyuko,
)
from ..model import Hikari_Model, Ship_Model


async def get_nation_list():
    try:
        msg = ''
        url = f'{hikari_config.yuyuko_url}/public/wows/encyclopedia/nation/list'
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, timeout=10)
        result = orjson.loads(resp.content)
        for nation in result['data']:
            msg: str = msg + f"{nation['cn']}：{nation['nation']}\n"
        return msg
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        logger.error(traceback.format_exc())


async def get_ship_name(hikari: Hikari_Model):
    """根据国家等级类型查船名"""
    msg = ''
    try:
        params = {
            'country': hikari.Input.ShipInfo.Ship_Nation,
            'level': hikari.Input.ShipInfo.Ship_Tier,
            'shipName': '',
            'shipType': hikari.Input.ShipInfo.Ship_Type,
            'groupType': 'default',
        }
        url = f'{hikari_config.yuyuko_url}/public/wows/encyclopedia/ship/search'
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=10)
        result = orjson.loads(resp.content)
        if result['data']:
            for ship in result['data']:
                msg += f"{ship['nameCn']}：{ship['nameEnglish']}\n"
            return hikari.success(msg)
        else:
            return hikari.failed('没有符合的船只')
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return hikari.error('请求超时了，请过会儿再尝试哦~')
    except PoolTimeout:
        await recreate_client_yuyuko()
        return hikari.error('连接池异常，请尝试重新查询~')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')


async def get_ship_byName(shipname: str) -> List:
    try:
        shipname_select_index = None
        result = shipname.split('.')
        if len(result) == 2 and result[1].isdigit():
            shipname = result[0]
            shipname_select_index = int(result[1])
        url = f'{hikari_config.yuyuko_url}/public/wows/encyclopedia/ship/search'
        params = {'country': '', 'level': '', 'shipName': shipname, 'shipType': '', 'groupType': 'default'}
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=10)
        result = orjson.loads(resp.content)
        List, select_List = [], []
        if result['code'] == 200 and result['data']:
            for each in result['data']:
                List.append(
                    Ship_Model(
                        Ship_Nation=each['country'],
                        Ship_Tier=each['level'],
                        Ship_Type=each['shipType'],
                        Ship_Name_Cn=each['nameCn'],
                        Ship_Name_English=each['nameEnglish'],
                        ship_Name_Numbers=each['nameEnglish'],
                        Ship_Id=each['shipId'],
                    )
                )
            if shipname_select_index and shipname_select_index <= len(List):
                select_List.append(List[shipname_select_index - 1])
                return select_List
            else:
                return List
        else:
            return None
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        logger.error(traceback.format_exc())
        return None


async def get_all_shipList():
    try:
        url = f'{hikari_config.yuyuko_url}/public/wows/encyclopedia/ship/search'
        params = {'country': '', 'level': '', 'shipName': '', 'shipType': '', 'groupType': 'default'}
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=10)
        result = orjson.loads(resp.content)
        if result['code'] == 200 and result['data']:
            return result['data']
        else:
            return None
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        return None


async def get_AccountIdByName(server: str, name: str) -> str:
    try:
        url = f'{hikari_config.yuyuko_url}/public/wows/account/search/{server}/user'
        params = {'userName': name, 'one': True}
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.post(url, json=params, timeout=10)
        result = orjson.loads(resp.content)
        if result['code'] == 200 and result['data']:
            return int(result['data'][0]['accountId'])
        else:
            return result['message']
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return '请求超时了，请过一会儿重试哦~'
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        logger.error(traceback.format_exc())
        return '好像出了点问题呢，可能是网络问题，如果重试几次还不行的话，请联系麻麻解决'


async def get_ClanIdByName(server: str, tag: str):
    try:
        url = f'{hikari_config.yuyuko_url}/public/wows/clan/search/{server}'
        params = {
            'tag': tag,
        }
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=10)
        result = orjson.loads(resp.content)
        if result['code'] == 200 and result['data']:
            return result['data']
        else:
            return None
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return None
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        logger.error(traceback.format_exc())
        return None


async def check_yuyuko_cache(server, id):
    try:
        yuyuko_cache_url = f'{hikari_config.yuyuko_url}/api/wows/cache/check'
        params = {'accountId': id, 'server': server}
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.post(yuyuko_cache_url, json=params, timeout=5)
        result = orjson.loads(resp.content)
        cache_data = {}
        if result['code'] == 201:
            if 'DEV' in result['data']:
                await get_wg_info(cache_data, 'DEV', result['data']['DEV'])
            elif 'PVP' in result['data']:
                tasks = []
                for key in result['data']:
                    tasks.append(asyncio.ensure_future(get_wg_info(cache_data, key, result['data'][key])))
                await asyncio.gather(*tasks)
            if not cache_data:
                return False
            data_base64 = b64encode(gzip.compress(orjson.dumps(cache_data))).decode()
            params['data'] = data_base64
            resp = await client_yuyuko.post(yuyuko_cache_url, json=params, timeout=5)
            result = orjson.loads(resp.content)
            logger.success(result)
            if result['code'] == 200:
                return True
            else:
                return False
        return False
    except PoolTimeout:
        await recreate_client_yuyuko()
        return False
    except Exception:
        logger.error('缓存上报失败')
        return False


async def get_wg_info(params, key, url):
    try:
        client_wg = await get_client_wg()
        resp = await client_wg.get(url, timeout=5, follow_redirects=True)
        wg_result = orjson.loads(resp.content)
        if resp.status_code == 200 and wg_result['status'] == 'ok':
            params[key] = resp.text
    except PoolTimeout:
        await recreate_client_wg()
        return
    except Exception:
        logger.error(f'wg请求异常,请配置代理后尝试,上报url：{url}')
        return


async def get_MyShipRank_yuyuko(params) -> int:
    try:
        url = 'https://api.wows.shinoaki.com/upload/numbers/data/upload/user/ship/rank'
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=5)
        result = orjson.loads(resp.content)
        if result['code'] == 200 and result['data']:
            if result['data']['ranking']:
                return result['data']['ranking']
            elif not result['data']['ranking'] and not result['data']['serverId'] == 'cn':
                ranking = await get_MyShipRank_Numbers(result['data']['httpUrl'], result['data']['serverId'])
                if ranking:
                    await post_MyShipRank_yuyuko(
                        result['data']['accountId'],
                        ranking,
                        result['data']['serverId'],
                        result['data']['shipId'],
                    )
                return ranking
            else:
                return None
        else:
            return None
    except PoolTimeout:
        await recreate_client_yuyuko()
        return None
    except Exception:
        logger.error(traceback.format_exc())
        return None


async def get_MyShipRank_Numbers(url, server) -> int:
    try:
        data = None
        client_default = await get_client_default()
        resp = await client_default.get(url, timeout=10)
        if resp.content:
            result = orjson.loads(resp.content)
            page_url = str(result['url']).replace('\\', '')
            nickname = str(result['nickname'])
            my_rank_url = f'{number_url_homes[server]}{page_url}'
            resp = await client_default.get(my_rank_url, timeout=10)
            soup = BeautifulSoup(resp.content, 'html.parser')
            data = soup.select_one(f'tr[data-nickname="{nickname}"]').select_one('td').string
        if data and data.isdigit():
            return data
        else:
            return None
    except PoolTimeout:
        await recreate_client_default()
        return None
    except Exception:
        logger.error(traceback.format_exc())
        return None


async def post_MyShipRank_yuyuko(accountId, ranking, serverId, shipId):
    try:
        url = 'https://api.wows.shinoaki.com/upload/numbers/data/upload/user/ship/rank'
        post_data = {
            'accountId': int(accountId),
            'ranking': int(ranking),
            'serverId': serverId,
            'shipId': int(shipId),
        }
        client_yuyuko = await get_client_yuyuko()
        await client_yuyuko.post(url, json=post_data, timeout=10)
        return
    except PoolTimeout:
        await recreate_client_yuyuko()
        return
    except Exception:
        logger.error(traceback.format_exc())
        return



import httpx
from httpx import AsyncClient, Request, Response
from loguru import logger

from .config import hikari_config
from .data_source import __version__


async def before_request(request: Request):
    logger.info(f'{request.method} {request.url}')


async def after_response(response: Response):
    logger.info(f'本次响应的状态码:{response.status_code} {response.http_version} {response.request}')


_client_yuyuko: AsyncClient = None
_client_wg: AsyncClient = None
_client_default: AsyncClient = None


async def create_client_yuyuko() -> AsyncClient:
    global _client_yuyuko
    _client_yuyuko = httpx.AsyncClient(
        headers={
            'Authorization': hikari_config.token,
            'accept': 'application/json',
            'Content-Type': 'application/json',
            'Yuyuko-Client-Type': f'{hikari_config.yuyuko_type};{__version__}',
        },
        event_hooks={
            'request': [
                before_request,
            ],
            'response': [
                after_response,
            ],
        },
        http2=hikari_config.http2,
    )
    logger.info('创建client_yuyuko')
    return _client_yuyuko


async def create_client_wg() -> AsyncClient:
    if hikari_config.proxy:
        proxy = {'https://': hikari_config.proxy}
    else:
        proxy = {}
    global _client_wg
    _client_wg = httpx.AsyncClient(proxies=proxy)
    logger.info('创建client_wg')
    return _client_wg


async def create_client_default() -> AsyncClient:
    global _client_default
    _client_default = httpx.AsyncClient()
    logger.info('创建client_default')
    return _client_default


async def get_client_yuyuko() -> AsyncClient:
    return _client_yuyuko if _client_yuyuko else await create_client_yuyuko()


async def get_client_wg() -> AsyncClient:
    return _client_wg if _client_wg else await create_client_wg()


async def get_client_default() -> AsyncClient:
    return _client_default if _client_default else await create_client_default()


async def recreate_client_yuyuko():
    _client_yuyuko = await get_client_yuyuko()
    logger.info('重新创建yuyuko连接池')
    await _client_yuyuko.aclose()
    _client_yuyuko = await create_client_yuyuko()


async def recreate_client_wg():
    _client_wg = await get_client_wg()
    logger.info('重新创建wg连接池')
    await _client_wg.aclose()
    _client_wg = await create_client_wg()


async def recreate_client_default():
    _client_default = await get_client_default()
    logger.info('重新创建default连接池')
    await _client_default.aclose()
    _client_default = await create_client_default()



import traceback
from asyncio.exceptions import TimeoutError

import orjson
from httpx import ConnectTimeout, PoolTimeout
from loguru import logger

from ..HttpClient_Pool import get_client_yuyuko, recreate_client_yuyuko
from ..model import Hikari_Model
from ..moudle.publicAPI import get_AccountIdByName
from ..moudle.wws_bind import get_DefaultBindInfo


async def get_BanInfo(hikari: Hikari_Model) -> Hikari_Model:
    """查询封禁匹配记录"""
    try:
        if hikari.Status == 'init':
            if hikari.Input.Search_Type == 3:
                hikari.Input.AccountId = await get_AccountIdByName(hikari.Input.Server, hikari.Input.AccountName)
                if not isinstance(hikari.Input.AccountId, int):
                    return hikari.error(f'{hikari.Input.AccountId}')
            else:
                bindResult = await get_DefaultBindInfo(hikari.Input.Platform, hikari.Input.PlatformId)
                if bindResult:
                    if bindResult['serverType'] == 'cn':
                        hikari.Input.AccountId = int(bindResult['accountId'])
                    else:
                        return hikari.error('目前仅支持国服查询')
                else:
                    return hikari.error('未查询到该用户绑定信息，请使用wws 查询绑定 进行检查')
        else:
            return hikari.error('当前请求状态错误')
        url = 'https://api.wows.shinoaki.com/public/wows/ban/cn/user'
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.post(url, json={'accountId': hikari.Input.AccountId}, timeout=10)
        result = orjson.loads(resp.content)
        hikari.Output.Yuyuko_Code = result['code']
        if result['code'] == 200 and result['data']:
            hikari = hikari.set_template_info('wws-ban.html', 900, 100)
            return hikari.success(result['data'])
        elif result['code'] == 404 and result['data']:
            hikari = hikari.set_template_info('wws-unban.html', 900, 100)
            return hikari.success(result['data'])
        else:
            return hikari.failed(f"{result['message']}")
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return hikari.error('请求超时了，请过会儿再尝试哦~')
    except PoolTimeout:
        await recreate_client_yuyuko()
        return hikari.error('连接池异常，请尝试重新查询~')
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error('wuwuwu出了点问题，请联系麻麻解决')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author         : yanyongyu
@Date           : 2021-03-12 13:42:43
@LastEditors    : yanyongyu
@LastEditTime   : 2021-11-01 14:05:41
@Description    : None
@GitHub         : https://github.com/yanyongyu
"""
__author__ = 'yanyongyu'
from contextlib import asynccontextmanager
from typing import AsyncIterator, Optional

from loguru import logger
from playwright.async_api import Browser, Error, Page, Playwright, async_playwright

_browser: Optional[Browser] = None
_playwright: Optional[Playwright] = None


async def init(use_browser, **kwargs) -> Browser:
    global _browser
    global _playwright
    _playwright = await async_playwright().start()
    try:
        _browser = await launch_browser(use_browser, **kwargs)
    except Error:
        await install_browser(use_browser)
        _browser = await launch_browser(use_browser, **kwargs)
    return _browser


async def launch_browser(use_browser, **kwargs) -> Browser:
    assert _playwright is not None, 'Playwright 没有安装'
    if use_browser == 'firefox':
        logger.info('使用 firefox 启动')
        return await _playwright.firefox.launch(**kwargs)

    else:
        # 默认使用 chromium
        logger.info('使用 chromium 启动')
        return await _playwright.chromium.launch(**kwargs)


async def get_browser(use_browser, **kwargs) -> Browser:
    return _browser if _browser and _browser.is_connected() else await init(use_browser, **kwargs)


@asynccontextmanager
async def get_new_page(use_browser, **kwargs) -> AsyncIterator[Page]:
    browser = await get_browser(use_browser)
    page = await browser.new_page(**kwargs)
    try:
        yield page
    finally:
        await page.close()


async def shutdown_browser():
    global _browser
    global _playwright
    if _browser:
        await _browser.close()
        _browser = None
    if _playwright:
        await _playwright.stop()  # type: ignore
        _playwright = None


async def install_browser(use_browser):
    import os
    import sys

    from playwright.__main__ import main

    logger.info('使用镜像源进行下载')
    os.environ['PLAYWRIGHT_DOWNLOAD_HOST'] = 'https://npmmirror.com/mirrors/playwright/'
    success = False

    if use_browser == 'firefox':
        logger.info('正在安装 firefox')
        sys.argv = ['', 'install', 'firefox']
    else:
        # 默认使用 chromium
        logger.info('正在安装 chromium')
        sys.argv = ['', 'install', 'chromium']
    try:
        logger.info('正在安装依赖')
        os.system('playwright install-deps')
        main()
    except SystemExit as e:
        if e.code == 0:
            success = True
    if not success:
        logger.error('浏览器更新失败, 请检查网络连通性')



import time
import traceback

import jinja2
from apscheduler.schedulers.background import BackgroundScheduler
from jinja2.exceptions import UndefinedError
from loguru import logger
from playwright.async_api import Error as playwright_Error
from pydantic import ValidationError

from .analyze import analyze_command
from .command_select import *  # noqa: F403
from .config import hikari_config, set_hikari_config  # noqa:F401
from .data_source import set_render_params, template_path
from .game.help import update_template
from .Html_Render import html_to_pic
from .model import Hikari_Model, Input_Model, UserInfo_Model

env = jinja2.Environment(loader=jinja2.FileSystemLoader(template_path), enable_async=True)
env.globals.update(
    time=time,
    abs=abs,
    enumerate=enumerate,
    int=int,
)


async def init_hikari(
    platform: str,
    PlatformId: str,
    command_text: str = None,
    GroupId: str = None,
    Ignore_List=[],  # noqa: B006
) -> Hikari_Model:
    """Hikari初始化

    Args:
        platform (str): 平台类型
        PlatformId (str): 平台ID
        command_text (str): 传入指令，不带wws
        GroupId (str): 群号,不配置无法使用部分分群功能
        Ignore_List(List):  禁用功能列表，通过import导入
    Returns:
        Hikari_Model: 可通过Hikari.Status和Hikari.Output.Data内数据判断是否输出
    """
    try:
        userinfo = UserInfo_Model(Platform=platform, PlatformId=PlatformId, GroupId=GroupId)
        input = Input_Model(Command_Text=command_text)
        hikari = Hikari_Model(UserInfo=userinfo, Input=input)
        hikari = await analyze_command(hikari)
        if not hikari.Status == 'init' or not hikari.Function:
            return hikari
        if hikari.Function in Ignore_List:
            return hikari.error('该功能已被禁用')
        hikari: Hikari_Model = await hikari.Function(hikari)
        return await output_hikari(hikari)
    except ValidationError:
        logger.error(traceback.format_exc())
        return Hikari_Model().error('参数校验错误，请联系开发者确认入参是否符合Model')
    except Exception:
        logger.error(traceback.format_exc())
        return Hikari_Model().error('Hikari-core顶层错误，请检查log')


async def callback_hikari(hikari: Hikari_Model) -> Hikari_Model:
    """回调wait状态的Hikari

    Args:
        hikari (Hikari_Model):前置或自行构造的Hikari_Model，可通过from hikari_core import Hikari_Model引入

    Returns:
        Hikari_Model: 可通过Hikari.Status和Hikari.Output.Data内数据判断是否输出
    """
    try:
        if not hikari.Status == 'wait':
            return hikari.error('当前请求状态错误，请确认是否为wait')
        if not hikari.Function:
            return hikari.error('缺少请求方法')
        hikari: Hikari_Model = await hikari.Function(hikari)
        return await output_hikari(hikari)

    except Exception:
        logger.error(traceback.format_exc())
        return Hikari_Model().error('Hikari-core顶层错误，请检查log')


async def output_hikari(hikari: Hikari_Model) -> Hikari_Model:
    """输出Hikari

    Args:
        hikari (Hikari_Model):前置或自行构造的Hikari_Model，可通过from hikari_core import Hikari_Model引入

    Returns:
        Hikari_Model: 可通过Hikari.Status和Hikari.Output.Data内数据判断是否输出
    """
    try:
        if (
            hikari.Status in ['success', 'wait']
            and hikari_config.auto_rendering
            and hikari.Output.Template
            and (isinstance(hikari.Output.Data, dict) or isinstance(hikari.Output.Data, list))  # noqa: PLR1701
        ):
            template = env.get_template(hikari.Output.Template)
            if hikari.Status == 'success':
                template_data = await set_render_params(hikari.Output.Data)
            elif hikari.Status == 'wait':
                template_data = await set_render_params(hikari.Input.Select_Data)
            content = await template.render_async(template_data)
            hikari.Output.Data = content
            hikari.Output.Data_Type = type(hikari.Output.Data)

            if hikari_config.auto_image:
                hikari.Output.Data = await html_to_pic(
                    content,
                    wait=0,
                    viewport={'width': hikari.Output.Width, 'height': hikari.Output.Height},
                    use_browser=hikari_config.use_broswer,
                )
                hikari.Output.Data_Type = type(hikari.Output.Data)
        return hikari
    except UndefinedError as e:
        logger.error(traceback.format_exc())
        return Hikari_Model().error(f'模板渲染错误，请将错误日志提交给开发者\n{e}')
    except playwright_Error as e:
        logger.error(traceback.format_exc())
        return Hikari_Model().error(f'playwright错误，请检查浏览器内核是否异常结束，可能是由于服务器版本过低，请升级至winserver2016+或改为firefox启动。\n{e}')
    except Exception as e:
        logger.error(traceback.format_exc())
        return Hikari_Model().error(f'Hikari-core顶层错误，请检查log\n{e}')


update_template()
scheduler = BackgroundScheduler(timezone='Asia/Shanghai')
scheduler.add_job(update_template, 'cron', hour='4,12')
scheduler.start()

# logger.add(
#    'hikari-core-logs/error.log',
#    rotation='00:00',
#    retention='1 week',
#    diagnose=False,
#    level='ERROR',
#    encoding='utf-8',
# )
# logger.add(
#    'hikari-core-logs/info.log',
#    rotation='00:00',
#    retention='1 week',
#    diagnose=False,
#    level='INFO',
#    encoding='utf-8',
# )
# logger.add(
#    'hikari-core-logs/warning.log',
#    rotation='00:00',
#    retention='1 week',
#    diagnose=False,
#    level='WARNING',
#    encoding='utf-8',
# )
